#include <string.h>
#include <stdlib.h>
#include "geosim.h"
#include "gspriv.h"
#include "ietypes.h"
#include "windfunc.h"
#include "palettew.h"
#include "iemisc.h"
#include "editrec.h"
#include "helpmsg.h"

#define MAX_COL 63
#define PAL_ROWS 16
#define PAL_COLS 16

/* from ie: */
extern GSColor IEInterface_Palette[];
extern char *IEColorlist[];
extern char *IEAlias[], *IEAcolor[];
extern boolean palwind;

/* from GIL: */
extern char Interface_Palette[];
extern char *Colorlist[];
extern char *Alias[], *Acolor[];

/* Internal Protos */
void SelectColor(int slot);
static void TempFieldsToColor(int colNum);
static void ColorToTempFields(int colNum);
void ResetTempCol();
static void AdjustButtons();
static void SetupList();
static void ShowList();
static void DisplayPalette(boolean);
static void PaletteSelect(int colNum, boolean select);
static int PointToColor(int x, int y);
char *colLabel(int slot);
void colText(GSlistHandle, int slot);
void colIn(int slot);
void colOut(void);
void colClick(int slot);
/**/

/* internal globals */
static GSlistHandle colList;
static GSsclistHandle colScroll;
static EditRec *colName;
static int numColors, curColor;
static BigString tempName;
static char tred, tgreen, tblue;
static int palx, paly, palwd, palht;
/**/

char *colLabel(int slot)
{
	if (slot < numColors)
		return Colorlist[slot];
	else
		return "ERROR!";
}

void colText(GSlistHandle l, int slot)
{
	if (slot == curColor)
		GSsetcolor(GSconvertcolor("curitem"));
	else
		GSsetcolor(GSconvertcolor("black"));
}

void colIn(int slot)
{
}

void colOut(void)
{
}

void colClick(int slot)
{
	SelectColor(slot);
}

void SelectColor(int slot)
{
	int old = curColor;
	
	curColor = slot;
	
	if (old >= 0) {
		if (strlen(tempName) > 0) { /* color is valid and 
									a color was selected */
			TempFieldsToColor(old);
			}
		GSdrawlistitem(colList, New, old);
		DisplayPalette(FALSE);
		PaletteSelect(old, FALSE);			
		}
		
	GSdrawlistitem(colList, New, curColor);
	PaletteSelect(curColor, TRUE);
	
	GSenablebutton("palettew", "colDelete", TRUE);
	ColorToTempFields(curColor);
	DrawNumField("colRedVal", tred);
	DrawNumField("colGreenVal", tgreen);
	DrawNumField("colBlueVal", tblue);
	AdjustButtons();
	GSenablebutton("palettew", "colDelete", TRUE);
}

void TempFieldsToColor(int colNum)
{
	free(Colorlist[colNum]);
	Colorlist[colNum] = malloc(strlen(tempName)+1);
	if (Colorlist[colNum] == NULL)
		GSelog(-1, "Out of memory for color.\n");
	strcpy(Colorlist[colNum], tempName);
	Interface_Palette[colNum*3] = tred;
	Interface_Palette[colNum*3+1] = tgreen;
	Interface_Palette[colNum*3+2] = tblue;
	GSPinit_palette(Interface_Palette, 256);
}

void ColorToTempFields(int colNum)
{
	strcpy(tempName, Colorlist[colNum]);
	tred = Interface_Palette[colNum*3];
	tgreen = Interface_Palette[colNum*3+1];
	tblue = Interface_Palette[colNum*3+2];
	UpdateEditRec(colName);
}

void ResetTempCol()
{
	tred = 0;
	tgreen = 0;
	tblue = 0;
	*tempName = '\0';
	colName->inspt = 0;
}

static void AdjustButtons()
{
	if (tred <= 0)
		GSdisablebutton("palettew", "Decred", TRUE);
	else
		GSenablebutton("palettew", "Decred", TRUE);
	if (tred >= MAX_COL)
		GSdisablebutton("palettew", "Incred", TRUE);
	else
		GSenablebutton("palettew", "Incred", TRUE);

	if (tgreen <= 0)
		GSdisablebutton("palettew", "Decgreen", TRUE);
	else
		GSenablebutton("palettew", "Decgreen", TRUE);
	if (tgreen >= MAX_COL)
		GSdisablebutton("palettew", "Incgreen", TRUE);
	else
		GSenablebutton("palettew", "Incgreen", TRUE);

	if (tblue <= 0)
		GSdisablebutton("palettew", "Decblue", TRUE);
	else
		GSenablebutton("palettew", "Decblue", TRUE);
	if (tblue >= MAX_COL)
		GSdisablebutton("palettew", "Incblue", TRUE);
	else
		GSenablebutton("palettew", "Incblue", TRUE);
}

void DrawPaletteWind()
{
	palwind = TRUE;
	SwapPalette(Interface_Palette, (char *) IEInterface_Palette,
				Colorlist, IEColorlist,
				Alias, IEAlias,
				Acolor, IEAcolor, NULL, NULL, NULL, NULL);
	GSPinit_palette(Interface_Palette, 256);
	GSgetdragareaspecs("palettew", "palDisp", 
			&palx, &paly, &palwd, &palht, NULL, NULL);
	GSdisablebutton("palettew", "colDelete", FALSE);
	

	GSdrawnamedwindow("palettew", NO_BSTORE, NO_POPUP);
	colName = NewEditRec(tempName, BIGSTRINGLEN, 1, SMALL,
			"palettew", "colNameEdit", c_identifier_filter, NULL);
	
	GSdisablemenuitem("Edit", "edit_palette");
	ResetTempCol();
	AdjustButtons();
	SetupList();
	if (numColors < 256)
		GSenablebutton("palettew", "colNew", TRUE);
	else
		GSdisablebutton("palettew", "colNew", TRUE);
		
	DisplayPalette(FALSE);
	SetCurrentEditRec(colName);
	UpdateEditRec(colName);
	DrawNumField("colRedVal", tred);
	DrawNumField("colGreenVal", tgreen);
	DrawNumField("colBlueVal", tblue);
	GSdisplayhelpmessage(COLOR_H);
	ShowVersion();
}

static void SetupList()
{
	for (numColors = 0; Colorlist[numColors] != NULL; numColors++);
	curColor = NO_ITEM;
	
	GSsclistsetup("palettew", "colPick",
					1,
					"pickLabel",
					0, 0,
					JUST_LEFT,
					colLabel,
					colText,
					colIn,
					colOut,
					colClick,
					"colScroll",
					GSconvertcolor("sclistcol"),
					GSconvertcolor("sclisttl"),
					GSconvertcolor("sclistbr"),
					numColors, 0,
					NULL,
					"colUpBtn", "colDownBtn",
					&colList, &colScroll);
	
	ShowList();
}

static void ShowList()
{
	int x, y, wd, ht;
	GSgetdragareaspecs("palettew", "colPick", &x, &y, &wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);
	GSdrawsclist(colScroll);
}

static void DisplayPalette(boolean erase)
{
	int colNum, x, y, wd = palwd/PAL_COLS, ht = palht/PAL_ROWS;
	
	GSsetcolor(GSconvertcolor("black"));
	GSwdrawrect(palx-1, paly-1, palwd+2, palht+2);
	
	if (erase)
		GSwfillrect(palx-1, paly-1, palwd+2, palht+2);
	
	for (colNum = 0; Colorlist[colNum] != NULL; colNum++) {
		if (colNum == curColor)
			PaletteSelect(colNum, TRUE);
		x = palx + ((colNum % PAL_ROWS)*wd);
		y = paly + ((colNum / PAL_ROWS)*ht);
		GSsetcolor(GSconvertcolor(Colorlist[colNum]));
		GSwfillrect(x+1, y+1, wd-2, ht-2);
		}
}

static void PaletteSelect(int colNum, boolean select)
{
	int x, y, wd = palwd/PAL_COLS, ht = palht/PAL_ROWS;
	x = palx + ((colNum % PAL_ROWS)*wd);
	y = paly + ((colNum / PAL_ROWS)*ht);
	GSsetcolor(GSconvertcolor(select? "white" : "black"));
	GSwdrawrect(x, y, wd, ht);
}

static int PointToColor(int x, int y) /* da-relative coords */
{
	return ((y/(palht/PAL_ROWS))*PAL_COLS) + (x/(palwd/PAL_COLS));
}

void RedrawPaletteWind()
{
	GSredrawnamedwindow("palettew");
	ShowList();
	UpdateEditRec(colName);
	DrawNumField("colRedVal", tred);
	DrawNumField("colGreenVal", tgreen);
	DrawNumField("colBlueVal", tblue);
	DisplayPalette(FALSE);
}

void RemovePaletteWind()
{
	palwind = FALSE;
	DisposEditRec(colName);	
	GSdismantlesclist(colScroll);
	GSremovenamedwindow("palettew");
	GSenablemenuitem("Edit", "edit_palette");
	SwapPalette(Interface_Palette, (char *) IEInterface_Palette,
				Colorlist, IEColorlist,
				Alias, IEAlias,
				Acolor, IEAcolor, NULL, NULL, NULL, NULL);
	GSPinit_palette(Interface_Palette, 256);
}

boolean col_btn(char *bname)
{
	if (bname[3] == 'U')
		GShandleupbutton(colScroll);
	else
		GShandledownbutton(colScroll);

	return TRUE;
}

boolean col_new(char *i)
{
	int old = curColor;
	
	if (strlen(tempName) > 0) {
		if (curColor == NO_ITEM) {
			Colorlist[numColors] = malloc(strlen(tempName)+1);
			curColor = numColors;
			numColors++;
			Colorlist[numColors] = NULL;
			TempFieldsToColor(curColor);
			GSsetsclistnumitems(colScroll, numColors, TRUE);
			}
		else
			TempFieldsToColor(curColor);
		}

	curColor = NO_ITEM;
	ResetTempCol();
	AdjustButtons();
	UpdateEditRec(colName);
	DrawNumField("colRedVal", tred);
	DrawNumField("colGreenVal", tgreen);
	DrawNumField("colBlueVal", tblue);
	DisplayPalette(TRUE);
	if (old != NO_ITEM)
		GSdrawlistitem(colList, New, old);
	return TRUE;
}

boolean col_delete(char *n)
{
	int i;
	
	GSdisablebutton("palettew", "colDelete", TRUE);
	if (curColor == NO_ITEM)
		return;
		
	for (i=curColor; i<numColors; i++) {
		Colorlist[i] = Colorlist[i+1];
		Interface_Palette[3*i] = Interface_Palette[3*(i+1)];
		Interface_Palette[3*i+1] = Interface_Palette[3*(i+1)+1];
		Interface_Palette[3*i+2] = Interface_Palette[3*(i+1)+2];
		}
	free(Colorlist[numColors]);
	Colorlist[numColors] = NULL;
	GSPinit_palette(Interface_Palette, 256);
	
	curColor = NO_ITEM;
	
	numColors--;
	GSsetsclistnumitems(colScroll, numColors, TRUE);
	ResetTempCol();
	AdjustButtons();
	UpdateEditRec(colName);
	DrawNumField("colRedVal", tred);
	DrawNumField("colGreenVal", tgreen);
	DrawNumField("colBlueVal", tblue);
	DisplayPalette(TRUE);
	
	return TRUE;
}

boolean col_red(char *name)
{
	if (name[0] == 'I') {
		tred++;
		if (tred == MAX_COL)
			GSdisablebutton("palettew", "Incred", TRUE);
		else if (tred == 1)
			GSenablebutton("palettew", "Decred", TRUE);
		}
	else {
		tred--;
		if (tred == 0)
			GSdisablebutton("palettew", "Decred", TRUE);
		else if (tred == MAX_COL-1)
			GSenablebutton("palettew", "Incred", TRUE);
		}
	DrawNumField("colRedVal", tred);
		
	return TRUE;
}

boolean col_green(char *name)
{
	if (name[0] == 'I') {
		tgreen++;
		if (tgreen == MAX_COL)
			GSdisablebutton("palettew", "Incgreen", TRUE);
		else if (tgreen == 1)
			GSenablebutton("palettew", "Decgreen", TRUE);
		}
	else {
		tgreen--;
		if (tgreen == 0)
			GSdisablebutton("palettew", "Decgreen", TRUE);
		else if (tgreen == MAX_COL-1)
			GSenablebutton("palettew", "Incgreen", TRUE);
		}
	DrawNumField("colGreenVal", tgreen);
		
	return TRUE;
}

boolean col_blue(char *name)
{
	if (name[0] == 'I') {
		tblue++;
		if (tblue == MAX_COL)
			GSdisablebutton("palettew", "Incblue", TRUE);
		else if (tblue == 1)
			GSenablebutton("palettew", "Decblue", TRUE);
		}
	else {
		tblue--;
		if (tblue == 0)
			GSdisablebutton("palettew", "Decblue", TRUE);
		else if (tblue == MAX_COL-1)
			GSenablebutton("palettew", "Incblue", TRUE);
		}
	DrawNumField("colBlueVal", tblue);
		
	return TRUE;
}

boolean col_pick(int x, int y, DragStatusType s)
{
	GShandlelist(colList, x, y, s);
	return TRUE;
}

boolean col_scroll(int x, int y, DragStatusType s)
{
	GShandlesclist(colScroll, x, y, s);
	return TRUE;
}

boolean col_name_edit(int x, int y, DragStatusType status)
{
	if ((status == MOUSE_CLICK) || (status == DRAG_FINAL))
		SetCurrentEditRec(colName);
	return TRUE;
}

boolean col_display(int x, int y, DragStatusType s)
{
	int colNum;
	
	if ((s == MOUSE_CLICK) || (s == DRAG_FINAL)) {
		colNum = PointToColor(x, y);
		if (colNum < numColors)
			SelectColor(colNum);
		}
	return TRUE;
}
